@@ -47,8 +47,6 @@
 
 extern GstTool *tool;
 
-extern gboolean updating;
-
 static void
 scrolled_window_scroll_bottom (GtkWidget *sw)
 {
@@ -68,6 +66,7 @@ on_network_admin_show (GtkWidget *w, gpointer user_data)
 
 	tool = user_data;
 	connection_init_gui (tool);
+	dns_search_init_gui (tool);
 	hosts_init_gui (tool);
 	profiles_table_create (tool);
 }
@@ -156,6 +155,30 @@ poll_connections_timeout (gpointer data)
 }
 #endif
 
+gboolean
+callbacks_button_press (GtkTreeView *treeview, GdkEventButton *event, gpointer gdata)
+{
+	GtkTreePath *path;
+	GtkItemFactory *factory;
+
+	factory = (GtkItemFactory *) gdata;
+
+	if (event->button == 3)
+	{
+		gtk_widget_grab_focus (GTK_WIDGET (treeview));
+		if (gtk_tree_view_get_path_at_pos (treeview, event->x, event->y, &path, NULL, NULL, NULL))
+		{
+			gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (treeview));
+			gtk_tree_selection_select_path (gtk_tree_view_get_selection (treeview), path);
+
+			gtk_item_factory_popup (factory, event->x_root, event->y_root,
+						event->button, event->time);
+		}
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void
 on_network_notebook_switch_page (GtkWidget *notebook, GtkNotebookPage *page,
 				 gint page_num, gpointer user_data)
@@ -333,8 +356,7 @@ filter_editable (GtkEditable *editable, const gchar *text, gint length,
 		s = g_strconcat (gtk_editable_get_chars (editable, 0, -1), text, NULL);
 		success = is_ip_text_ok (s);
 		g_free (s);
-	}
-	else {
+	} else {
 		while ((i < length) && (string_ok)) {
 			if (is_char_ok (text[i], rules))
 				i++;
@@ -348,48 +370,10 @@ filter_editable (GtkEditable *editable, const gchar *text, gint length,
 			success = FALSE;
 	}
 
-	if (!success) {
-		gdk_beep ();
-	
+	if (!success)
 		gtk_signal_emit_stop_by_name (GTK_OBJECT (editable), "insert_text");
-/*		if (s) {
-			gtk_editable_insert_text (editable, s, l, pos);
-			}*/
-	}
 }
 
-/* yeah, I don't like this formatting either */
-static const char *hint_entry[][3] = { {
-	"hostname", "general_help", 		
-	N_("The name of this computer")
-},{
-	"smbdesc", "general_help", 
-	N_("A short description of your computer")
-},{
-	"workgroup", "general_help",
-	N_("The Windows Networking workgroup for your network")
-},{
-	"winsserver", "general_help", 
-	N_("The IP address of your WINS server")
-},{
-	"domain", "dns_help",     
-	N_("The DNS domain for your computer")
-},{
-	"dns_list", "dns_help",
-	N_("A list of your DNS servers' IP addresses"),
-},{
-	"search_list", "dns_help",
-	N_("A list of domains where hosts will be searched") 
-},{
-	"ip", "hosts_help",
-	N_("The IP address of this host") 
-},{
-	"alias", "hosts_help",
-	N_("Aliases for this host, one per line") 
-},{
-	NULL
-} };
-
 static GHashTable *help_hash;
 
 void
@@ -406,41 +390,23 @@ init_editable_filters (GstDialog *dialog)
 		{ "network_connection_other_gateway", EF_ALLOW_IP },
 		{ "network_connection_plip_local_ip", EF_ALLOW_IP },
 		{ "network_connection_plip_remote_ip", EF_ALLOW_IP },
-		{ NULL,          EF_ALLOW_NONE }
-	};
-
-	struct tmp s1[] = {
-		{ "dns_list",    EF_ALLOW_ENTER },
-		{ "search_list", EF_ALLOW_ENTER | EF_ALLOW_TEXT },
-		{ "alias",       EF_ALLOW_ENTER | EF_ALLOW_TEXT },
+		{ "ip_address",          EF_ALLOW_IP },
+      		{ "ip_netmask",          EF_ALLOW_IP },
+		{ "ip_gateway",          EF_ALLOW_IP },
+		{ "ppp_dns1",            EF_ALLOW_IP },
+		{ "ppp_dns2",            EF_ALLOW_IP },
+		{ "ptp_address",         EF_ALLOW_IP },
+		{ "ptp_remote_address",  EF_ALLOW_IP },
+		{ "dns_server_entry",    EF_ALLOW_IP },
+		{ "search_domain_entry", EF_ALLOW_ENTER | EF_ALLOW_TEXT },
 		{ NULL,          EF_ALLOW_NONE }
 	};
 
 	for (i = 0; s[i].name; i++)
-		connect_editable_filter (gst_dialog_get_widget (dialog, s[i].name), s[i].rule);
-
-#warning FIXME
-	return;
-
-	for (i = 0; s1[i].name; i++) {
-		GtkTextBuffer *buffer;
-		GtkWidget *w = gst_dialog_get_widget (dialog, s1[i].name);
-
-		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (w));
-		g_signal_connect (G_OBJECT (buffer), "insert_text",
+		g_signal_connect (G_OBJECT (gst_dialog_get_widget (dialog, s[i].name)),
+				  "insert_text",
 				  G_CALLBACK (filter_editable),
-				  GINT_TO_POINTER (s1[i].rule));
-	}
-}
-
-void
-init_hint_entries (void)
-{
-	int i;
-	help_hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-	for (i=0; hint_entry[i][0]; i++)
-		g_hash_table_insert (help_hash, (char *)hint_entry[i][0], hint_entry[i]);
+				  GINT_TO_POINTER (s[i].rule));
 }
 
 gint
@@ -471,7 +437,7 @@ on_connection_add_clicked (GtkWidget *w, gpointer null)
 	GtkWidget *druid_window = gst_dialog_get_widget (tool->main_dialog, "network_connection_window");
 	GnomeDruid *druid = GNOME_DRUID (gst_dialog_get_widget (tool->main_dialog, "network_connection_druid"));
 
-	network_druid_new (druid, tool, GST_CONNECTION_UNKNOWN);
+	network_druid_new (druid, druid_window, tool, GST_CONNECTION_UNKNOWN);
 	gtk_widget_show_all (druid_window);
 }
 
@@ -510,7 +476,27 @@ on_connection_delete_clicked (GtkWidget *w, gpointer null)
 void
 on_connection_configure_clicked (GtkWidget *w, gpointer null)
 {
-	connection_configure (connection_list_get_active ());
+	GstConnection *cxn = connection_list_get_active ();
+	
+	connection_configure (cxn);
+}
+
+void
+on_connection_popup_add_activate (gpointer callback_data, guint action, GtkWidget *widget)
+{
+	on_connection_add_clicked (callback_data, NULL);
+}
+
+void
+on_connection_popup_configure_activate (gpointer callback_data, guint action, GtkWidget *widget)
+{
+	on_connection_configure_clicked (callback_data, NULL);
+}
+
+void
+on_connection_popup_delete_activate (gpointer callback_data, guint action, GtkWidget *widget)
+{
+	on_connection_delete_clicked (callback_data, NULL);
 }
 
 static void
@@ -523,14 +509,14 @@ activate_directive_cb (GstDirectiveEntry *entry)
 	g_free (entry->report_sign);
 }
 
-void
-on_connection_activate_clicked (GtkWidget *w, gpointer null)
+static void
+on_connection_activate_clicked (GtkWidget *w, GtkTreePath *path, gpointer null)
 {
 	GstConnection *cxn;
 	gchar *sign, *file;
 	gboolean modified = gst_dialog_get_modified (tool->main_dialog);
 
-	cxn = connection_list_get_active ();
+	cxn = connection_list_get_by_path (path);
 	connection_activate (cxn, TRUE);
 
 	file = (cxn->file)? cxn->file: cxn->dev;
@@ -561,14 +547,14 @@ deactivate_directive_cb (GstDirectiveEntry *entry)
 	g_free (entry->report_sign);
 }
 
-void
-on_connection_deactivate_clicked (GtkWidget *w, gpointer null)
+static void
+on_connection_deactivate_clicked (GtkWidget *w, GtkTreePath *path, gpointer null)
 {
 	GstConnection *cxn;
 	gchar *sign, *file;
 	gboolean modified = gst_dialog_get_modified (tool->main_dialog);
 
-	cxn = connection_list_get_active ();
+	cxn = connection_list_get_by_path (path);
 	connection_activate (cxn, FALSE);
 	
 	file = (cxn->file)? cxn->file: cxn->dev;
@@ -612,7 +598,7 @@ on_samba_use_toggled (GtkWidget *w, gpointer null)
 						 GTK_MESSAGE_INFO,
 						 GTK_BUTTONS_OK,
 						 _("You don't have SMB support installed. Please install "
-						   "SMB support\nin the system to enable windows networking."));
+						   "SMB support in the system to enable windows networking."));
 
 		gtk_window_set_title (GTK_WINDOW (dialog), _("SMB support missing."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -665,9 +651,9 @@ callbacks_check_hostname_hook (GstDialog *dialog, gpointer data)
 
 	if (strcmp (hostname_new, hostname_old))
 	{
-		gchar *text = _("The host name has changed. This will prevent you\n"
-				"from launching new applications, and so you will\n"
-				"have to log in again.\n\nContinue anyway?");
+		gchar *text = _("The host name has changed. This will prevent you "
+				"from launching new applications, and so you will "
+				"have to log in again. Continue anyway?");
 		GtkWidget *message;
 
 		message = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
@@ -681,10 +667,11 @@ callbacks_check_hostname_hook (GstDialog *dialog, gpointer data)
 
 		switch (res) {
 		case GTK_RESPONSE_OK:
-			gtk_entry_set_text (GTK_ENTRY (entry), hostname_old);
-		case GTK_RESPONSE_CANCEL:
 			g_free (hostname_old);
 			return TRUE;
+		case GTK_RESPONSE_CANCEL:
+			gtk_entry_set_text (GTK_ENTRY (entry), hostname_old);
+			return FALSE;
 		default:
 			g_free (hostname_old);
 			return FALSE;
@@ -708,19 +695,20 @@ callbacks_check_dialer (GtkWindow *window, GstTool *tool)
 {
 	gboolean has_dialer;
 	
-	has_dialer = connection_list_has_dialer (tool);
+	has_dialer = (gboolean) g_object_get_data (G_OBJECT (tool), "dialinstalled");
+	
 	if (!has_dialer)
 	{
-		gchar *text = _("wvdial could not be found on your system.\n"
-				"You need to install wvdial, or the PPP (modem)\n"
+		gchar *text = _("wvdial could not be found on your system. "
+				"You need to install wvdial, or the PPP (modem) "
 				"connections will not activate.");
 		GtkWidget *message;
 
-		gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
-					GTK_DIALOG_MODAL,
-					GTK_MESSAGE_WARNING,
-					GTK_BUTTONS_OK,
-					text);
+		message = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
+						  GTK_DIALOG_MODAL,
+						  GTK_MESSAGE_WARNING,
+						  GTK_BUTTONS_OK,
+						  text);
 
 		gtk_dialog_run (GTK_DIALOG (message));
 		gtk_widget_destroy (message);
@@ -734,12 +722,13 @@ callbacks_check_dialer_hook (GstDialog *dialog, gpointer data)
 	gboolean has_dialer;
 
 	tool = GST_TOOL (data);
+	
 	has_dialer = connection_list_has_dialer (tool);
 	if (!has_dialer)
 	{
-		gchar *text = _("wvdial could not be found on your system.\n"
-				"You need to install wvdial, or the PPP (modem)\n"
-				"connections will not activate.\n\nContinue anyway?");
+		gchar *text = _("wvdial could not be found on your system. "
+				"You need to install wvdial, or the PPP (modem) "
+				"connections will not activate. Continue anyway?");
 		gint res;
 		GtkWidget *message;
 
@@ -764,9 +753,9 @@ callbacks_check_dialer_hook (GstDialog *dialog, gpointer data)
 static gboolean
 callbacks_disabled_gatewaydev_warn (GstTool *tool, GstConnection *cxn, gboolean *ignore_enabled)
 {
-	gchar *text = _("The default gateway device is not activated. This\n"
-			"will prevent you from connecting to the Internet.\n"
-			"\nContinue anyway?");
+	gchar *text = _("The default gateway device is not activated. This "
+			"will prevent you from connecting to the Internet. "
+			"Continue anyway?");
 	gint res;
 	GtkWidget *message;
 
@@ -819,9 +808,9 @@ callbacks_check_manual_gatewaydev (GstTool *tool)
 		case GST_CONNECTION_ERROR_STATIC:
 		{
 			GtkWidget *dialog;
-			gchar *txt = _("The default gateway device is missing gateway\n"
-				       "information. Please provide this information to\n"
-				       "proceed, or choose another default gateway device.\n");
+			gchar *txt = _("The default gateway device is missing gateway "
+				       "information. Please provide this information to "
+				       "proceed, or choose another default gateway device.");
 
 			dialog = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
 							 GTK_DIALOG_MODAL,
@@ -865,9 +854,9 @@ gboolean
 callbacks_tool_not_found_hook (GstTool *tool, GstReportLine *rline, gpointer data)
 {
 	if (! strcmp (rline->argv[0], "redhat-config-network-cmd")) {
-		gchar *text = _("The program redhat-config-network-cmd could not\n"
-				"be found. This could render missing connections\n"
-				"under the connections tab. Please install the\n"
+		gchar *text = _("The program redhat-config-network-cmd could not "
+				"be found. This could render missing connections "
+				"under the connections tab. Please install the "
 				"redhat-config-network rpm package to avoid this.");
 		GtkWidget *message;
 
@@ -884,143 +873,146 @@ callbacks_tool_not_found_hook (GstTool *tool, GstReportLine *rline, gpointer dat
 	return TRUE;
 }
 
-
 /* Connection dialog callbacks */
 
 void
-on_connection_list_clicked (GtkWidget *w, gpointer data)
+on_connection_toggled (GtkWidget *w, gchar *path_str, gpointer data)
 {
-	GtkTreePath *path;
-	GtkTreeViewColumn *column;
-	GtkTreeModel *model;
+	GtkTreeView *treeview = GTK_TREE_VIEW (data);
+	GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
 	GtkTreeIter iter;
-	GList *column_list;
-	gint ncol;
-	GdkPixbuf *stat_icon;
 	GstConnection *cxn;
-	GtkWidget *dialog;
 	gchar *txt;
+	GtkWidget *dialog;
 
-	cxn = connection_list_get_active ();
-	column_list = gtk_tree_view_get_columns (GTK_TREE_VIEW (w));
-	gtk_tree_view_get_cursor (GTK_TREE_VIEW (w), &path, &column);
-							  
-	ncol = g_list_index (column_list, column) + 1;
+	gtk_tree_model_get_iter (model, &iter, path);
+	cxn = connection_list_get_by_path (path);
 
-	if (ncol == CONNECTION_LIST_COL_STAT_PIX)
-	{
-		model = gtk_tree_view_get_model (GTK_TREE_VIEW (w));
-		gtk_tree_model_get_iter (model, &iter, path);
-		
-		if (!cxn->enabled)
-		{
-			if (cxn->type != GST_CONNECTION_LO) {
-				txt = g_strdup_printf (_("Do you want to enable interface %s?"), cxn->dev);
-				dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, txt);
-				if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
-					on_connection_activate_clicked (w, NULL);
+	if (!cxn->enabled) {
+		if (cxn->type != GST_CONNECTION_LO) {
+			txt = g_strdup_printf (_("Do you want to enable interface %s?"), cxn->dev);
+			dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, txt);
+			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+				on_connection_activate_clicked (GTK_WIDGET (treeview), path, NULL);
 
-				gtk_widget_destroy (dialog);
-				g_free (txt);
-			}
+			gtk_widget_destroy (dialog);
+			g_free (txt);
 		}
-		else
-		{
-			if (cxn->type != GST_CONNECTION_LO) {
-				txt = g_strdup_printf (_("Do you want to disable interface %s?"), cxn->dev);
-				dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, txt);
-				if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
-					on_connection_deactivate_clicked (w, NULL);
+	} else {
+		if (cxn->type != GST_CONNECTION_LO) {
+			txt = g_strdup_printf (_("Do you want to disable interface %s?"), cxn->dev);
+			dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, txt);
+			if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+				on_connection_deactivate_clicked (GTK_WIDGET (treeview), path,  NULL);
 
-				gtk_widget_destroy (dialog);
-				g_free (txt);
-			}
+			gtk_widget_destroy (dialog);
+			g_free (txt);
 		}
 	}
-	g_list_free (column_list);
+	
 	gtk_tree_path_free (path);
 }
 
 void
-on_connection_ok_clicked (GtkWidget *w, GstConnection *cxn)
+on_connection_ok_clicked (GtkWidget *w, gpointer data)
 {
+	GtkWidget *window = gst_dialog_get_widget (tool->main_dialog, "connection_config_dialog");
+	GstConnection *cxn = g_object_get_data (G_OBJECT (window), "connection_data");
+	gboolean standalone = (g_object_get_data (G_OBJECT (window), "standalone") != NULL);
+	gboolean add_to_list = !standalone;
 
-	if (cxn->modified) {
-		if (connection_config_save (cxn))
-			gtk_widget_destroy (cxn->window);
+	if ((cxn->modified) && (connection_config_save (cxn, FALSE))) {
 		cxn->creating = FALSE;
 		gst_dialog_modify (tool->main_dialog);
-	} else
-		gtk_widget_destroy (cxn->window);
-}
-
-void
-on_connection_cancel_clicked (GtkWidget *w, GstConnection *cxn)
-{
-	gtk_widget_destroy (cxn->window);
-
-	if (cxn->creating) {
-		connection_list_remove (cxn);
-		connection_free (cxn);
-		gst_dialog_modify (tool->main_dialog);
 	}
+
+	if (standalone) {
+		gtk_widget_set_sensitive (window, FALSE);
+		connection_save_to_node (cxn, gst_xml_doc_get_root (tool->config));
+		gtk_signal_emit_by_name (GTK_OBJECT (tool->main_dialog), "apply", tool);
+		gtk_main_quit ();
+	}
+
+	gtk_widget_hide (window);
+}
+
+gboolean
+on_connection_delete_event (GtkWidget *window, GdkEvent *event, gpointer data)
+{
+	on_connection_cancel_clicked (window, NULL);
+	return TRUE;
 }
 
 void
-on_connection_config_dialog_destroy (GtkWidget *w, GstConnection *cxn)
+on_connection_cancel_clicked (GtkWidget *w, gpointer data)
 {
-	g_object_unref (G_OBJECT (cxn->xml));
-	cxn->xml = NULL;
+	GtkWidget *window = gst_dialog_get_widget (tool->main_dialog, "connection_config_dialog");
 
-	cxn->window = NULL;
-}
+	if (g_object_get_data (G_OBJECT (window), "standalone") != NULL)
+		gtk_main_quit ();
 
-gint
-on_connection_config_dialog_delete_event (GtkWidget *w, GdkEvent *evt, GstConnection *cxn)
-{
-	return FALSE;
+	gtk_widget_hide (window);
 }
 
 void
-on_connection_modified (GtkWidget *w, GstConnection *cxn)
+on_connection_modified (GtkWidget *w, gpointer data)
 {
+	GtkWidget *dialog = gst_dialog_get_widget (tool->main_dialog, "connection_config_dialog");
+	GstConnection *cxn = g_object_get_data (G_OBJECT (dialog), "connection_data");
+
 	connection_set_modified (cxn, TRUE);
 }
 
 void
-on_wvlan_adhoc_toggled (GtkWidget *w, GstConnection *cxn)
+on_ppp_autodetect_modem_clicked (GtkWidget *widget, gpointer data)
 {
-/* FIXME: implement on_wvlan_adhoc_toggled*/
-}
-
-void
-on_ppp_autodetect_modem_clicked (GtkWidget *widget, GstConnection *cxn)
-{
-	gchar *dev = connection_autodetect_modem ();
+	GtkWidget *autodetect_button = gst_dialog_get_widget (tool->main_dialog, "ppp_autodetect_modem");
+	GtkWidget *devices_combo = gst_dialog_get_widget (tool->main_dialog, "ppp_serial_port_g");
+	GtkWidget *dialog = gst_dialog_get_widget (tool->main_dialog, "connection_config_dialog");
 	GtkWidget *w;
+	gchar *dev;
+	GdkCursor *cursor;
 
-	if (strcmp (dev, "") == 0) {
+	/* give some feedback to let know the user that the tool is busy */
+	gtk_widget_set_sensitive (autodetect_button, FALSE);
+	gtk_widget_set_sensitive (devices_combo, FALSE);
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_window_set_cursor (GTK_WIDGET (dialog)->window, cursor);
+	gdk_cursor_unref (cursor);
+
+	dev = connection_autodetect_modem ();
+
+	if ((!dev) || (strcmp (dev, "") == 0)) {
 		w = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
 					    _("Could not autodetect modem device, check that this is not busy and that it's correctly attached"));
 		gtk_dialog_run (GTK_DIALOG (w));
 		gtk_widget_destroy (w);
 	} else {
-		w = glade_xml_get_widget (cxn->xml, "ppp_serial_port");
+		w = gst_dialog_get_widget (tool->main_dialog, "ppp_serial_port");
 		gtk_entry_set_text (GTK_ENTRY (w), dev);
 	}
+
+	if (dev)
+		g_free (dev);
+
+	/* remove the user feedback */
+	gtk_widget_set_sensitive (autodetect_button, TRUE);
+	gtk_widget_set_sensitive (devices_combo, TRUE);
 	
-	g_free (dev);
+	gdk_window_set_cursor (GTK_WIDGET (dialog)->window, NULL);
 }
 
 void
-on_ppp_update_dns_toggled (GtkWidget *w, GstConnection *cxn)
+on_ppp_update_dns_toggled (GtkWidget *w, gpointer data)
 {
 	gboolean active;
-	GtkWidget *ppp_update_dns = glade_xml_get_widget (cxn->xml, "ppp_update_dns");
-	GtkWidget *ppp_dns1_label = glade_xml_get_widget (cxn->xml, "ppp_dns1_label");
-	GtkWidget *ppp_dns2_label = glade_xml_get_widget (cxn->xml, "ppp_dns2_label");
-	GtkWidget *ppp_dns1 = glade_xml_get_widget (cxn->xml, "ppp_dns1");
-	GtkWidget *ppp_dns2 = glade_xml_get_widget (cxn->xml, "ppp_dns2");
+	GtkWidget *ppp_update_dns = gst_dialog_get_widget (tool->main_dialog, "ppp_update_dns");
+	GtkWidget *ppp_dns1_label = gst_dialog_get_widget (tool->main_dialog, "ppp_dns1_label");
+	GtkWidget *ppp_dns2_label = gst_dialog_get_widget (tool->main_dialog, "ppp_dns2_label");
+	GtkWidget *ppp_dns1 = gst_dialog_get_widget (tool->main_dialog, "ppp_dns1");
+	GtkWidget *ppp_dns2 = gst_dialog_get_widget (tool->main_dialog, "ppp_dns2");
 	
 	active = GTK_TOGGLE_BUTTON (ppp_update_dns)->active;
 
@@ -1031,15 +1023,140 @@ on_ppp_update_dns_toggled (GtkWidget *w, GstConnection *cxn)
 }
 
 gboolean
-on_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event, GstConnection *cxn)
+on_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
-	GtkWidget *netmask_widget = glade_xml_get_widget (cxn->xml, "ip_netmask");
+	GtkWidget *netmask_widget = gst_dialog_get_widget (tool->main_dialog, "ip_netmask");
 
-	connection_check_netmask_gui (cxn, widget, netmask_widget);
+	connection_check_netmask_gui (widget, netmask_widget);
 
         return FALSE;
 }
 
+gchar*
+on_volume_format_value (GtkWidget *widget, gdouble value, gpointer data)
+{
+	gchar *string;
+	
+	if (value == 0)
+		string = g_strdup_printf (_("Silent"));
+	else if (value == 1)
+		string = g_strdup_printf (_("Low"));
+	else if (value == 2)
+		string = g_strdup_printf (_("Medium"));
+	else if (value == 3)
+		string = g_strdup_printf (_("Loud"));
+
+	return string;
+}
+
+/* Dns tab callbacks */
+void 
+on_dns_search_add_button_clicked (GtkWidget *button, gpointer gdata)
+{
+	GtkWidget *treeview;
+	GtkWidget *entry;
+
+	if (strcmp (gtk_widget_get_name (GTK_WIDGET (button)),
+		    "dns_server_add_button") == 0) {
+		treeview = gst_dialog_get_widget (tool->main_dialog,
+					       "dns_list");
+		entry = gst_dialog_get_widget (tool->main_dialog,
+					       "dns_server_entry");
+	}
+	else if (strcmp (gtk_widget_get_name (GTK_WIDGET (button)),
+			 "search_domain_add_button") == 0) {
+		treeview = gst_dialog_get_widget (tool->main_dialog,
+					       "search_list");
+		entry = gst_dialog_get_widget (tool->main_dialog,
+					       "search_domain_entry");
+	}
+
+	dns_search_list_append (treeview, gtk_entry_get_text (
+					GTK_ENTRY (entry)));
+}
+
+void 
+on_dns_search_del_button_clicked (GtkWidget *widget, gpointer gdata)
+{
+	GtkWidget *treeview;
+	GtkWidget *entry;
+
+	if (strcmp (gtk_widget_get_name (GTK_WIDGET (widget)),
+		    "dns_server_del_button") == 0) {
+		treeview = gst_dialog_get_widget (tool->main_dialog,
+						  "dns_list");
+		entry = gst_dialog_get_widget (tool->main_dialog,
+					       "dns_server_entry");
+	}
+	else if (strcmp (gtk_widget_get_name (GTK_WIDGET (widget)),
+			 "search_domain_del_button") == 0) {
+		treeview = gst_dialog_get_widget (tool->main_dialog,
+						  "search_list");
+		entry = gst_dialog_get_widget (tool->main_dialog,
+					       "search_domain_entry");
+	}
+	else if (strcmp (gtk_widget_get_name (GTK_WIDGET (widget)),
+			 "dns_list") == 0) {
+		treeview = widget;
+		entry = gst_dialog_get_widget (tool->main_dialog,
+					       "dns_server_entry");
+	}
+	else if (strcmp (gtk_widget_get_name (GTK_WIDGET (widget)),
+			 "search_list") == 0) {
+		treeview = widget;
+		entry = gst_dialog_get_widget (tool->main_dialog,
+					       "search_domain_entry");
+	}
+
+	dns_search_list_remove (treeview, entry);
+}
+
+void
+on_dns_search_entry_changed (GtkWidget *widget, gpointer gdata)
+{
+	GtkWidget *list;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreeSelection *selection;
+	gboolean valid;
+	gchar const *text;
+	gchar *buf;
+
+	list = (GtkWidget *) gdata;
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
+	text = gtk_entry_get_text (GTK_ENTRY (widget));
+	
+	if ((!gst_dns_search_is_in_list (list, text)) &&
+	    (strlen (text) > 0))
+		gtk_tree_selection_unselect_all (selection);
+	else {
+		model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
+		valid = gtk_tree_model_get_iter_first (model, &iter);
+
+		while (valid) {
+			gtk_tree_model_get (model, &iter, 0, &buf, -1);
+
+			if (strcmp (buf, text) == 0) {
+				g_free (buf);
+				gtk_tree_selection_select_iter (selection, &iter);
+				break;
+			}
+
+			g_free (buf);
+			valid = gtk_tree_model_iter_next (model, &iter);
+		}
+	}
+
+	gst_dns_search_update_sensitivity (list);
+	gtk_widget_show_all (list);
+}
+
+void
+on_dns_search_popup_del_activate (gpointer callback_data, guint action, GtkWidget *widget)
+{
+	on_dns_search_del_button_clicked (callback_data, NULL);
+}
 
 /* Hosts tab callbacks */
 
@@ -1051,16 +1168,12 @@ gst_hosts_unselect_all (void)
 	GstStatichostUI  *ui;
 	gboolean          valid;
 
-	updating = TRUE;
-
 	ui = (GstStatichostUI *)g_object_get_data (G_OBJECT (tool), STATICHOST_UI_STRING);
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (ui->list));
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (ui->list));
 
 	gtk_tree_selection_unselect_all (select);
 	gst_ui_text_view_clear (GTK_TEXT_VIEW (ui->alias));
-
-	updating = FALSE;
 }
 
 static void
@@ -1076,8 +1189,6 @@ gst_hosts_select_row (const gchar *ip_str)
 	if (!ip_str || strlen (ip_str) == 0)
 		return;
 
-	updating = TRUE;
-
 	ui = (GstStatichostUI *)g_object_get_data (G_OBJECT (tool), STATICHOST_UI_STRING);
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (ui->list));
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (ui->list));
@@ -1089,7 +1200,6 @@ gst_hosts_select_row (const gchar *ip_str)
 		if (!strcmp (buf, ip_str)) {
 			gtk_tree_selection_select_iter (select, &iter);
 
-			updating = FALSE;
 			g_free (buf);
 
 			return;
@@ -1098,8 +1208,6 @@ gst_hosts_select_row (const gchar *ip_str)
 		g_free (buf);
 		valid = gtk_tree_model_iter_next (model, &iter);
 	}
-
-	updating = FALSE;
 }	
 
 static void
@@ -1109,10 +1217,8 @@ gst_hosts_clear_entries (void)
 
 	ui = (GstStatichostUI *)g_object_get_data (G_OBJECT (tool), STATICHOST_UI_STRING);
 
-	updating = TRUE;
 	gtk_editable_delete_text (GTK_EDITABLE (ui->ip), 0, -1);
 	gst_ui_text_view_clear (GTK_TEXT_VIEW (ui->alias));
-	updating = FALSE;
 }
 
 static char *
@@ -1136,9 +1242,6 @@ on_hosts_ip_changed (GtkEditable *ip, gpointer not_used)
 {
 	const gchar *ip_str;
 
-	if (updating)
-		return;
-
 	gst_hosts_update_sensitivity ();
 
 	/* Get the texts */
@@ -1150,6 +1253,7 @@ on_hosts_ip_changed (GtkEditable *ip, gpointer not_used)
 void
 on_hosts_alias_changed (GtkTextBuffer *w, gpointer not_used)
 {
+	const gchar *ip_str;
 	char *s;
 	GstStatichostUI *ui;
 	GtkTreeModel    *model;
@@ -1159,22 +1263,25 @@ on_hosts_alias_changed (GtkTextBuffer *w, gpointer not_used)
 	if (!gst_tool_get_access (tool))
 		return;
 
-	if (updating)
-		return;
-
 	ui = (GstStatichostUI *)g_object_get_data (G_OBJECT (tool), STATICHOST_UI_STRING);
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (ui->list));
+	ip_str = gtk_editable_get_chars (GTK_EDITABLE (ui->ip), 0, -1);
 
-	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (ui->list));
-	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
-		s = fixup_text_list (ui->alias);
+	if (gst_hosts_ip_is_in_list (ip_str)) {
+		select = gtk_tree_view_get_selection (GTK_TREE_VIEW (ui->list));
+		if (gtk_tree_selection_get_selected (select, &model, &iter)) {
+			s = fixup_text_list (ui->alias);
+			
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    STATICHOST_LIST_COL_IP, gtk_editable_get_chars (GTK_EDITABLE (ui->ip), 0, -1),
+					    STATICHOST_LIST_COL_ALIAS, s,
+					    -1);
 
-		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-				    STATICHOST_LIST_COL_IP, gtk_editable_get_chars (GTK_EDITABLE (ui->ip), 0, -1),
-				    STATICHOST_LIST_COL_ALIAS, s,
-				    -1);
+			gst_dialog_modify (tool->main_dialog);
+			g_free (s);
 
-		g_free (s);
+
+		}
 	}
 }
 
@@ -1200,9 +1307,7 @@ on_hosts_add_clicked (GtkWidget * button, gpointer user_data)
 void
 on_hosts_delete_clicked (GtkWidget * button, gpointer user_data)
 {
-	gchar *txt, *ip, *alias;
-	GtkWidget *dialog;
-	gint res;
+	gchar *ip, *alias;
 	GstStatichostUI *ui;
 
 	g_return_if_fail (gst_tool_get_access (tool));
@@ -1212,20 +1317,6 @@ on_hosts_delete_clicked (GtkWidget * button, gpointer user_data)
 	if (!hosts_list_get_selected (&ip, &alias))
 		return;
 
-	txt = g_strdup_printf (_("Are you sure you want to delete the aliases for %s?"), ip);
-	dialog = gtk_message_dialog_new (GTK_WINDOW (tool->main_dialog),
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_QUESTION,
-					 GTK_BUTTONS_YES_NO,
-					 txt);
-
-	res = gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
-	g_free (txt);
-
-	if (res != GTK_RESPONSE_YES)
-		return;
-
 	hosts_list_remove (tool, ip);
 	gst_dialog_modify (tool->main_dialog);
 
@@ -1233,6 +1324,12 @@ on_hosts_delete_clicked (GtkWidget * button, gpointer user_data)
 	gst_hosts_unselect_all ();
 }
 
+void
+on_hosts_popup_del_activate (gpointer callback_data, guint action, GtkWidget *widget)
+{
+	on_hosts_delete_clicked (callback_data, NULL);
+}
+
 /* Network druid callbacks */
 gboolean
 on_network_druid_hide (GtkWidget *w, gpointer data)
@@ -1269,6 +1366,7 @@ on_network_druid_page_prepare (GnomeDruidPage *druid_page, GnomeDruid *druid, gp
 	gchar *next_default_focus[] = {
 		NULL,
 		"connection_type_modem_option",
+		"network_connection_wireless_device_entry",
 		"network_connection_other_config_type",
 		"network_connection_plip_local_ip",
 		"network_connection_ppp_phone",
@@ -1277,7 +1375,8 @@ on_network_druid_page_prepare (GnomeDruidPage *druid_page, GnomeDruid *druid, gp
 		NULL
 	};
 
-	g_return_if_fail (druid_data != NULL);
+	if (druid_data == NULL)
+		return;
 	
 	g_signal_stop_emission_by_name (druid_page, "prepare");
 	network_druid_check_page (druid, druid_data->current_page);
@@ -1286,6 +1385,8 @@ on_network_druid_page_prepare (GnomeDruidPage *druid_page, GnomeDruid *druid, gp
 		gtk_widget_grab_focus (gst_dialog_get_widget (druid_data->tool->main_dialog,
 							      next_default_focus [druid_data->current_page]));
 	}
+
+	network_druid_set_window_title (druid);
 }
 
 void
@@ -1305,7 +1406,7 @@ on_network_druid_finish (GnomeDruidPage *druid_page, GnomeDruid *druid, gpointer
 	GstConnection *cxn = network_druid_get_connection_data (druid);
 	xmlNodePtr root = gst_xml_doc_get_root (tool->config);
 	GtkWidget *window = gst_dialog_get_widget (tool->main_dialog, "network_connection_window");
-	
+
 	g_return_if_fail (cxn != NULL);
 
 	if (g_object_get_data (G_OBJECT (druid), "standalone") == NULL) {
@@ -1315,7 +1416,6 @@ on_network_druid_finish (GnomeDruidPage *druid_page, GnomeDruid *druid, gpointer
 
 		gst_dialog_modify (tool->main_dialog);
 
-                /*	scrolled_window_scroll_bottom (gst_dialog_get_widget (tool->main_dialog, "connection_list_sw"));*/
 		network_druid_clear (druid, FALSE);
 		gtk_widget_hide (window);
 	} else {
@@ -1369,7 +1469,7 @@ on_network_druid_ip_address_focus_out (GtkWidget *widget, GdkEventFocus *event,
 							"network_connection_other_ip_mask");
 	NetworkDruidData *druid_data = g_object_get_data (G_OBJECT (druid), "data");
 
-	connection_check_netmask_gui (druid_data->cxn, widget, mask_widget);
+	connection_check_netmask_gui (widget, mask_widget);
 
         return FALSE;
 }
@@ -1467,12 +1567,18 @@ on_network_profile_option_selected (GtkWidget *widget, gpointer data)
 	GtkTreeModel *connection_model = gtk_tree_view_get_model (GTK_TREE_VIEW (connection_table));
 	GtkWidget *statichost_table = gst_dialog_get_widget (tool->main_dialog, "statichost_list");
 	GtkTreeModel *statichost_model = gtk_tree_view_get_model (GTK_TREE_VIEW (statichost_table));
+	GtkWidget *dns_table = gst_dialog_get_widget (tool->main_dialog, "dns_list");
+	GtkTreeModel *dns_model = gtk_tree_view_get_model (GTK_TREE_VIEW (dns_table));
+	GtkWidget *search_table = gst_dialog_get_widget (tool->main_dialog, "search_list");
+	GtkTreeModel *search_model = gtk_tree_view_get_model (GTK_TREE_VIEW (search_table));
 	xmlNodePtr profile = (xmlNodePtr) data;
 	
 	profile_set_active (profile, tool);
 
-	gtk_list_store_clear (GTK_LIST_STORE (connection_model));	
-	gtk_list_store_clear (GTK_LIST_STORE (statichost_model));	
+	gtk_list_store_clear (GTK_LIST_STORE (connection_model));
+	gtk_list_store_clear (GTK_LIST_STORE (statichost_model));
+	gtk_list_store_clear (GTK_LIST_STORE (dns_model));
+	gtk_list_store_clear (GTK_LIST_STORE (search_model));
 	
 	transfer_profile_to_gui (tool, NULL);
 }
