@@ -78,9 +78,15 @@ typedef enum {
 	ACTIVATION_DOWN
 } ActivationType;
 
+enum {
+	CONNECTION_POPUP_NONE,
+	CONNECTION_POPUP_ADD,
+	CONNECTION_POPUP_SEPARATOR,
+	CONNECTION_POPUP_CONFIGURE,
+	CONNECTION_POPUP_DELETE
+};
+
 typedef struct {
-	GtkWidget *window;
-	GladeXML *xml;
 	xmlNode *node;
 	
 	GstConnectionType type;
@@ -114,10 +120,7 @@ typedef struct {
 	gchar *gateway;
 
 	/* Wavelan */
-	gboolean adhoc;
-	gchar *session_id;
-	gint channel;
-	gfloat frequency;
+	gchar *essid;
 
 	/* PPP */
 	gchar *phone_number;
@@ -174,7 +177,7 @@ void           connection_list_select_connection (GstConnection *cxn);
 void           connection_list_clear      (GstTool *tool);
 
 gboolean connection_config_save (GstConnection*, gboolean);
-void     connection_check_netmask_gui (GstConnection*, GtkWidget*, GtkWidget*);
+void     connection_check_netmask_gui (GtkWidget*, GtkWidget*);
 void     connection_set_modified (GstConnection*, gboolean);
 
 gchar *connection_find_new_device (xmlNodePtr, GstConnectionType);
