@@ -28,7 +28,7 @@
 #include <gnome.h>
 #include <gtk/gtk.h>
 
-#include "xst.h"
+#include "gst.h"
 
 #include "table.h"
 #include "groups-table.h"
@@ -36,16 +36,16 @@
 #include "callbacks.h"
 #include "user-group-xml.h"
 
+extern GstTool *tool;
+
+GtkWidget *groups_table;
+
 GroupsTableConfig groups_table_config [] = {
 	{ N_("Group"),	TRUE,	TRUE},
 	{ N_("GID"),	TRUE,	FALSE},
 	{NULL}
 };
 
-extern XstTool *tool;
-
-GtkWidget *groups_table;
-
 static void
 add_group_columns (GtkTreeView *treeview)
 {
@@ -60,7 +60,7 @@ add_group_columns (GtkTreeView *treeview)
 	{
 		renderer = gtk_cell_renderer_text_new ();
 
-		column = gtk_tree_view_column_new_with_attributes (i->name,
+		column = gtk_tree_view_column_new_with_attributes (_(i->name),
 		                                                   renderer,
 		                                                   "text",
 		                                                   j,
@@ -89,10 +89,11 @@ create_groups_table (void)
 {
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
+	GtkItemFactory *item_factory;
 	
 	model = create_groups_model ();
 	
-	groups_table = xst_dialog_get_widget (tool->main_dialog, "groups_table");
+	groups_table = gst_dialog_get_widget (tool->main_dialog, "groups_table");
 	gtk_tree_view_set_model (GTK_TREE_VIEW (groups_table), model);
 	
 	g_object_unref (model);
@@ -102,16 +103,21 @@ create_groups_table (void)
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (groups_table));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
+	item_factory = popup_item_factory_create (groups_table);
+	
 	g_signal_connect (G_OBJECT (selection), "changed",
 			  G_CALLBACK (on_table_clicked),
 			  (gpointer) groups_table);
+	g_signal_connect (G_OBJECT (groups_table), "button_press_event",
+			  G_CALLBACK (on_table_button_press),
+			  (gpointer) item_factory);
 }
 
 static char*
 groups_table_value (xmlNodePtr node, gchar *key)
 {
         g_return_val_if_fail (node != NULL, NULL);
-        return xst_xml_get_child_content (node, key);
+        return gst_xml_get_child_content (node, key);
 }
 
 static GroupTreeItem*
@@ -135,8 +141,10 @@ populate_groups_table (void)
 	
 	g_return_if_fail (model != NULL);
 	g_return_if_fail (root != NULL);
+
+	gtk_tree_store_clear (GTK_TREE_STORE (model));
 	
-	for (group = xst_xml_element_find_first (root, "group"); group != NULL; group = xst_xml_element_find_next (group, "group"))
+	for (group = gst_xml_element_find_first (root, "group"); group != NULL; group = gst_xml_element_find_next (group, "group"))
 	{
 		if (check_node_visibility (group))
 		{
